<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<gui name="parley"  version="12">

<MenuBar>
    <Menu name="file" >
        <Action append="open_merge" name="file_ghns" />
        <Action append="open_merge" name="file_open_downloaded" />
        <Separator/>
        <Action name="file_export" />
        <Separator/>
        <Action name="file_properties" />
        <Separator/>
        <Merge/>
    </Menu>
  <Merge/>
  <Menu name="settings" >
  </Menu>
 </MenuBar>
 <ToolBar noMerge="true" noEdit="true" deleted="true" name="mainToolBar">
  <text>Main Toolbar</text>
 </ToolBar>
 <ToolBar name="editorToolBar">
  <text>Editor Toolbar</text>
  <Action name="file_close"/>
  <Merge/>
 </ToolBar>
 <ToolBar name="practiceToolBar">
  <text>Practice Toolbar</text>
  <Merge/>
 </ToolBar>
 <ToolBar name="statisticsToolBar">
  <text>Statistics Toolbar</text>
  <Action name="file_close"/>
  <Merge/>
  <Separator/>
  <DefineGroup name="practice" />
 </ToolBar>
 <ToolBar name="practiceSummaryToolBar">
  <text>Practice Summary Toolbar</text>
  <Action name="file_close"/>
  <Merge/>
  <Separator/>
  <DefineGroup name="practice" />
 </ToolBar>
</gui>
