<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<gui name="practicesummary" version="5">

<MenuBar>
   <Menu name="file" >
   <text>&amp;File</text>
   <Action name="document_edit" />
   <Action name="file_close" />
  </Menu>
  <Menu name="learning" >
   <text>&amp;Practice</text>
   <Action name="practice_configure" />
   <Action name="practice_export" />
   <Action name="practice_start" />
  </Menu>
</MenuBar>

 <ToolBar name="practiceSummaryToolBar">
  <text>Practice Summary Toolbar</text>
  <Action name="document_edit" />
  <Action name="practice_start" group="practice" />
  <Action name="practice_export" group="practice" />
 </ToolBar>
</gui>
